import { AutoMap } from '@automapper/classes';
import {
  IsOptional,
  IsString,
  IsBoolean,
  IsDate,
  IsInt,
  ValidateIf,
  IsArray,
  IsEnum,
  IsNumber,
  ValidateNested,
} from 'class-validator';
import { Frequency } from '../entities/frequency.enum';
import { Transform, Type } from 'class-transformer';
import { AddressType } from '../entities/address.enum';

export class CreateAdRequest {
  @IsString()
  @IsOptional()
  @AutoMap()
  uuid?: string;

  @IsString()
  @AutoMap()
  userUuid: string;

  @ValidateIf((ad) => (ad.passenger ? false : true))
  @IsOptional()
  @IsBoolean()
  @AutoMap()
  driver?: boolean;

  @ValidateIf((ad) => (ad.driver ? false : true))
  @IsOptional()
  @IsBoolean()
  @AutoMap()
  passenger?: boolean;

  @Transform(({ value }) => Frequency[value], { toClassOnly: true })
  @IsEnum(Frequency)
  @AutoMap()
  frequency: Frequency;

  @Type(() => Date)
  @IsDate()
  @AutoMap()
  fromDate: Date;

  @Type(() => Date)
  @IsDate()
  @AutoMap()
  toDate: Date;

  @IsDate()
  @IsOptional()
  @AutoMap()
  monTime?: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  tueTime?: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  wedTime?: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  thuTime?: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  friTime?: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  satTime?: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  sunTime?: string;

  @IsInt()
  @IsOptional()
  @AutoMap()
  monMargin?: number;

  @IsInt()
  @IsOptional()
  @AutoMap()
  tueMargin?: number;

  @IsInt()
  @IsOptional()
  @AutoMap()
  wedMargin?: number;

  @IsInt()
  @IsOptional()
  @AutoMap()
  thuMargin?: number;

  @IsInt()
  @IsOptional()
  @AutoMap()
  friMargin?: number;

  @IsInt()
  @IsOptional()
  @AutoMap()
  satMargin?: number;

  @IsInt()
  @IsOptional()
  @AutoMap()
  sunMargin?: number;

  @ValidateIf((ad) => (ad.passenger ? false : true))
  @IsOptional()
  @IsInt()
  @AutoMap()
  seatsDriver?: number;

  @ValidateIf((ad) => (ad.driver ? false : true))
  //@Transform(({ value }) => (value ? value : 0), { toClassOnly: true })
  @IsOptional()
  @IsInt()
  @AutoMap()
  seatsPassenger?: number;

  @IsOptional()
  @IsBoolean()
  @AutoMap()
  strict?: boolean;

  @Type(() => Date)
  @IsDate()
  @IsOptional()
  @AutoMap()
  createdAt?: Date;

  @Type(() => Date)
  @IsDate()
  @IsOptional()
  @AutoMap()
  updatedAt?: Date;

  @ValidateNested({ each: true })
  @Transform(
    ({ value }) => {
      console.log('in dto');
      console.log(value);
      return value;
    },
    { toClassOnly: true },
  )
  @Type(() => AddressDTO)
  @AutoMap(() => [AddressDTO])
  addresses: AddressDTO[];
}

class AddressDTO {
  @IsNumber()
  @AutoMap()
  position: number;

  @IsNumber()
  @AutoMap()
  lon: number;

  @IsNumber()
  @AutoMap()
  lat: number;

  @IsString()
  @IsOptional()
  @AutoMap()
  name?: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  houseNumber?: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  street?: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  locality: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  postalCode: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  country: string;

  @Transform(({ value }) => AddressType[value], { toClassOnly: true })
  @AutoMap()
  type: AddressType;
}
